# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Chooser.
# 
# Chooser is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Chooser is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Chooser.  If not, see <http://www.gnu.org/licenses/>.

import copy
from observer_context import *

class Command:
	def __init__(self, h):
		self.history = h
	def redo(self):
		self.history.append(self)
	def undo(self):
		self.history.rewind(self)
	def possible_redo(self):
		return True
	def possible_undo(self):
		return True

class MacroCommand(Command):
	def __init__(self, h):
		Command.__init__(self, h)
		self.commands = []
	def execute(self):
		for command in self.commands:
			command.execute()
	def unexecute(self):
		for command in reversed(self.commands):
			command.unexecute()
	def possible_redo(self):
		for command in self.commands:
			if not command.possible_redo():
				#print command.__class__.__name__
				return False
		return True
	def possible_undo(self):
		for command in self.commands:
			if not command.possible_undo():
				return False
		return True
	def append(self, command):
		self.commands.append(command)
class ReversedCommand(Command):
	def __init__(self, command):
		#print "reversed"
		self.command = command
	def execute(self):
		self.command.unexecute()
	def unexecute(self):
		self.command.execute()
	def possible_redo(self):
		return self.command.possible_undo(self)
	def possible_undo(self):
		return self.command.possible_redo(self)
class History(Command, Context):
	def __init__(self):
		Context.__init__(self)
		self.commands = []
		self.pointer = 0
	def redo(self):
		command = self.commands[self.pointer][0]
		if command.possible_redo():
			command.execute()
			self.pointer += 1
			self.notify()
		else:
			self.notify("Unable to complete the command!")
	def undo(self):
		command = self.commands[self.pointer - 1][0]
		if command.possible_redo():
			self.pointer -= 1
			command.unexecute()
			self.notify()
		else:
			self.notify("Unable to complete the command!")
	def possible_redo(self):
		return self.pointer < len(self.commands)
	def possible_undo(self):
		return self.pointer > 0
	def append(self, command):
# 		com = copy.copy(command)
		com = command
		self.commands[self.pointer:] = [[com, command]]
		self.redo()
	def rewind(self, command):
		while(self.commands[self.pointer-1][1] != command):
			self.undo()
		self.undo()
class Storage:
	def __init__(self):
		self.data = None
	def set(self, data):
		self.data = data
	def get(self):
		return self.data
