# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Chooser.
# 
# Chooser is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Chooser is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Chooser.  If not, see <http://www.gnu.org/licenses/>.

from Tkinter import *
import copy
from colorer import Colorer
BIT_SHIFT = 4
class HyperlinkManager:
	prefix = 'hyper-'

	def __init__(self, text):

		self.text = text

#        self.text.tag_config("hyper", foreground="blue", underline=0)

		self.text.tag_bind("hyper", "<Enter>", self._enter)
		self.text.tag_bind("hyper", "<Leave>", self._leave)
		self.text.tag_bind("hyper", "<Button-1>", self._click)
#		self.text.tag_bind("hyper", "<Shift-Button-1>", self._shift_click)

		self.reset()

	def reset(self):
		self.links = {}

	def add(self, action,  tag_end):
		# add an action to the manager.  returns tags to use in
		# associated text widget
		tag = "%s%s" % (self.prefix,  tag_end)
		self.links[tag] = action
		return "hyper", tag

	def _enter(self, event):
		self.text.config(cursor="hand2")

	def _leave(self, event):
		self.text.config(cursor="")
	def _click(self, event):
#		print dir(event)
		for tag in self.text.tag_names(CURRENT):
			if tag.startswith(self.prefix):
				self.links[tag](shift = event.state & BIT_SHIFT )
				return

class SelectManager(HyperlinkManager):
	def __init__(self,  text):
		self.tags = []
		HyperlinkManager.__init__(self,  text)
		self.plen = len(self.prefix)
		self.last = None
		self.colorer = Colorer()
	def colorise(self,  word):
		color = self.colorer.color(word)
		self._colorise(word,  color)
	def highlight(self,  word):
		if self.last:
#			print self.last.tag,  self.last.word,  self.last.lemma
			color = self.colorer.color(self.last)
			self._colorise(self.last,  color)
		self._colorise(word,  self.colorer.current())
		self.see(word)
#		self.last = word
		self.last = copy.deepcopy(word)
	def see(self,  word):
		tag  = word.tags()[0]
		tag = self.prefix + tag
		self.text.see(tag + '.first')
	def _colorise(self,  word,  color): #TODO: Do it better!
		for tag in word.tags():
			tag = self.prefix + tag
			self.text.tag_config(tag,  foreground = color)
	def reset(self):
		HyperlinkManager.reset(self)
